////////////////////////////////////////////////////////////////
//
// Copyright (C) 2006 Affymetrix, Inc.
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License 
// (version 2.1) as published by the Free Software Foundation.
// 
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
// for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
//
////////////////////////////////////////////////////////////////

/**
/// @file   apt-tsv-join.cpp
/// @brief  Main for merging multiple tab separated text files.
*/

#include "TsvJoin.h"

using namespace std;

/**
\page apt-tsv-join MANUAL: apt-tsv-join (apt-1.8.0)

\p apt-tsv-join will take multiple tab separated files and merge them into a single file.

\section QuickStart Quick Start
To merge several files using the same key, use:

\code
apt-tsv-join -o data.txt -key=column-name -out-file=output.txt file1.txt file2.txt [...]
\endcode

Note that the first file will be streamed through line by line, while the later files will
be indexed. Thus if you have only one large file, you probably want to specify this file first.

The output columns will be in the same order as the input file order and the order within each
of the input files. 

Currently, only the first file can have the key duplicated. The key must be unique in all
the other files to be joined.

\subsection manualOptions Options:

\verbatim
apt-tsv-join - Merge multiple tab separated text files into a single file.
Usage:
   apt-tsv-join -o data.txt -k column-name file1.txt file2.txt [...]

options:
   -k, --key              Name of the column to use to join the
                          various files. 
   -o, --out-file         Output file to send the merged results to. 
   -p, --prepend-filename Prepend the filename to the column header. 
       --version          Display version information. 
   -h, --help             Print help message. 
\endverbatim

*/

int main (int argc, char* argv[])
{
  const string version ("apt-1.8.0");
  const string cvsId ("$Id: apt-tsv-join.cpp,v 1.11 2007/05/23 17:55:45 awilli Exp $");

  try
  {
    tsvJoin join (argc, argv, version, cvsId);
    join.run();
  }

  catch (exception& e)
  {
    // Verbose::out (1, e.what()); // errAbort() writes the error message.
    return 1;
  }
  return 0;
}
