/*
 *  This file is part of BGX, the Bayesian Gene eXpression program.
 *  Copyright (c) 2008 Ernest Turro <ernest.turro@ic.ac.uk>
 *
 *  BGX is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  BGX is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 
#include <algorithm>
#include <map>
#include <ctype.h>
#include <cstdlib>
#include <cstring>
#ifdef WIN32
 #include <io.h>
#else
  #include <sys/stat.h>
#endif
#include <dirent.h>
#include <stdio.h>
#include <string>
#include "file/TsvFile/TsvFile.h"
#include "file/TsvFile/PgfFile.h"
#include <iostream>
#include <fstream>
#include <iomanip>

using namespace std;
using namespace affx;

int main(int argc, char **argv) {

  if(argc < 3) {
    cout << "Usage: filterduplicates path_to_xmap_tsv path_to_new_filtered_xmap_tsv\n";
    exit(1);
  }

  if(strcmp(argv[1], argv[2]) == 0) {
    cerr << "Please specify different file names for the old and new tsv files.\n";
    exit(1);
  }

  char str[1024];

  char *tempfile = tmpnam(NULL);
  ofstream ofs(tempfile);
  // Try opening X:Map TSV file and output commented headers
  ifstream ifs(argv[1]);
  if(ifs.is_open()) {
    while (ifs.good()) {
      ifs.getline(str, 1024);
      if(str[0] == '#') 
        ofs << str << endl;
      else break;
    }
  } else {
    printf("Error: couldn't open transcripthit TSV file \"%s\"\n",argv[1]);
    exit(1);
  }
  ifs.close();

  // Try opening X:Map TSV file now for parsing
  TsvFile* tsv = new TsvFile();
  if ((tsv->open(string(argv[1])))!=TSV_OK) {
    printf("Error: couldn't open transcripthit TSV file \"%s\"\n",argv[1]);
    delete tsv;
    exit(1);
  }


  // Make bindings to TSV slots
  int transcript_id, probe_id, gc_count;
  string stable_id, probe_sequence;
  tsv->bind(0,"transcript_id",&transcript_id,TSV_BIND_REQUIRED);
  tsv->bind(0,"stable_id",&stable_id,TSV_BIND_REQUIRED);
  tsv->bind(1,"probe_id",&probe_id,TSV_BIND_REQUIRED);
  tsv->bind(1,"probe_sequence",&probe_sequence,TSV_BIND_REQUIRED);
  tsv->bind(1,"gc_count",&gc_count,TSV_BIND_REQUIRED);

  vector< vector<int> > probeIdSets;
  vector< vector<string> > identicalPS;
  while(tsv->nextLevel(0)==TSV_OK){
    vector<int> pr;
    vector<int> gcc;
    vector<string> seqs;
    while (tsv->nextLevel(1)==TSV_OK) {
      seqs.push_back(probe_sequence);
      pr.push_back(probe_id);
      gcc.push_back(gc_count);
    }
    vector< vector<int> >::iterator viit;
    if((viit = find(probeIdSets.begin(), probeIdSets.end(), pr)) == probeIdSets.end()) {
      probeIdSets.push_back(pr);
      vector<string> vs; vs.push_back(stable_id);identicalPS.push_back(vs);
      ofs << transcript_id << "\t" << stable_id << endl;
      for(int i=0; i < pr.size(); i++) {
        ofs << "\t" << pr[i] << "\t" << gcc[i] << "\t" << seqs[i] << endl;
      }
    } else {
      identicalPS[viit-probeIdSets.begin()].push_back(stable_id);
    }
  }

  ofs.close();
  tsv->close();
  delete tsv;


  /* ####### */
  ofs.open(argv[2]);
  // Try opening X:Map TSV file and output commented headers
  ifs.open(tempfile);
  if(ifs.is_open()) {
    while (ifs.good()) {
      ifs.getline(str, 1024);
      if(str[0] == '#') 
        ofs << str << endl;
      else break;
    }
  } else {
    printf("Error: couldn't open transcripthit TSV file \"%s\"\n",argv[1]);
    exit(1);
  }
  ifs.close();

  // Try opening X:Map TSV file now for parsing
  tsv = new TsvFile();
  if ((tsv->open(string(tempfile)))!=TSV_OK) {
    printf("Error: couldn't open transcripthit TSV file \"%s\"\n",tempfile);
    delete tsv;
    exit(1);
  }


  // Make bindings to TSV slots
  tsv->bind(0,"transcript_id",&transcript_id,TSV_BIND_REQUIRED);
  tsv->bind(0,"stable_id",&stable_id,TSV_BIND_REQUIRED);
  tsv->bind(1,"probe_id",&probe_id,TSV_BIND_REQUIRED);
  tsv->bind(1,"probe_sequence",&probe_sequence,TSV_BIND_REQUIRED);
  tsv->bind(1,"gc_count",&gc_count,TSV_BIND_REQUIRED);

  int tc=0;
  while(tsv->nextLevel(0)==TSV_OK){
    ofs << transcript_id << "\t";
    for(int j=0; j < identicalPS[tc].size(); j++) {
      ofs << identicalPS[tc][j];
      if(j < identicalPS[tc].size() -1) ofs << "+";
    }
    ofs << endl;
    while (tsv->nextLevel(1)==TSV_OK) {
      ofs << "\t" << probe_id << "\t" << gc_count << "\t" << probe_sequence << endl;
    }
    tc++;
  }

  ofs.close();
  tsv->close();
  delete tsv;


  /* ########### */

  return 0;
}
